"""Memo application for Firefox OS"""

from browser import doc, local_storage, html
import json

store = local_storage.storage

# Memos are stored as the value of store['brython-memo']
# The value is the JSON dump of a list
# memos[i] is a dict with keys : title, content

memos = []

def show_memos(*args):
    global memos
    try:
        memos = json.loads(store['brython-memo'])
    except KeyError:
        # for the first visit
        memos = [{'title':'Welcome',
            'content':'Enter memo title and content. The memo will be saved '\
                'automatically when going back to the home page'}]
    
    section = html.SECTION(role="region")

    # header with application name (Memos)
    # and an icon to create a new memo    
    header = html.HEADER()
    menu = html.MENU(Type="toolbar")
    new_memo = html.BUTTON(html.SPAN("add", Class="icon icon-add"), Id=-1)
    # click on the + sign will create a new memo
    new_memo.bind('click', edit_memo)
    menu <= new_memo    
    header <= menu

    header <= html.H1("Memos")

    section <= header

    # scrollable list of existing memos
    article = html.ARTICLE(Class="content scrollable")
    memo_list = html.SECTION()
    memo_list.setAttribute("data-type", "list")
    items = html.UL()
    for num, memo in enumerate(memos):
        # The P element inside the link has an attribute "Id" set to the rank
        # of the memo in the memos list
        # It will be used in the callback function to identify the memo
        # the user wants to edit
        link = html.A(html.P(memo['title'], Id=num), href="#")
        link.bind('click', edit_memo)
        items <= html.LI(link)
    memo_list <= items
    
    # add a blank line, otherwise scrolling doesn't show the last item
    memo_list <= html.LI(' ')
    
    article <= memo_list
    section <= article

    # erase window and insert section
    doc.body.clear()
    doc <= section
    
def edit_memo(ev):
    title, content = "", ""
    num = int(ev.target.id)
    editing = num !=-1

    if editing:
        # editing existing memo
        item = memos[num]
        title = item["title"]
        content = item["content"]

    # application screen
    section = html.SECTION(role="region", Class="skin-organic")

    # header
    header = html.HEADER()
    header <= html.INPUT(Type="hidden", Id="memo_num", value=num)

    btn_back = html.BUTTON(html.SPAN('back',Class="icon icon-back"))
    btn_back.bind('click', show_memos)
    header <= btn_back

    form = html.FORM(action="#")

    # field for memo title
    t_entry = html.INPUT(Type="text", Id="memo_title", placeholder="title", 
            required="required")
    if title:
        t_entry.value = title
    t_entry.bind('change', save_memo)
    
    form <= t_entry
    
    form <= html.BUTTON("Remove text", Type="reset")

    header <= form
    section <= header

    t_content = html.TEXTAREA(content, placeholder="content",
        Id="memo_content", required="required", Class="memo")

    t_content.bind('change', save_memo)
    
    section <= t_content
    
    if editing:
        # add toolbar with icon to delete the memo
        toolbar = html.DIV(role="toolbar")
        delete_btn = html.BUTTON("delete", Class="action-icon delete")
        delete_btn.bind('click', confirm_delete)
        toolbar <= html.UL(html.LI(delete_btn))
        
        # add empty UL to align the first one to the left
        toolbar <= html.UL()

        section <= toolbar

    doc.body.clear()
    doc <= section
    
def save_memo(ev):
    num = int(doc["memo_num"].value)
    
    # ignore if memo has no title
    title = doc['memo_title'].value
    if not title:
        return
    
    rec = {'title':doc['memo_title'].value,
            'content':doc['memo_content'].value
        }
    if num==-1: 
        # new record
        memos.append(rec)
        # update value of hidden field "memo_num" for subsequent changes
        doc["memo_num"].value = len(memos)-1
    else:
        # update
        memos[num] = rec

    # save memos in local storage
    store['brython-memo'] = json.dumps(memos)

def confirm_delete(ev):
    """Page to confirm deletion of a memo"""
    num = int(doc["memo_num"].value)
    
    form = html.FORM(role="dialog")
    form.setAttribute("data-type","confirm")
    
    section = html.SECTION()
    section <= html.H1('Confirmation')
    txt = memos[num]['title']
    section <= html.P('Are you sure you want to delete memo "'+txt+'" ?')
    form <= section
    
    menu = html.MENU()
    cancel_btn = html.BUTTON("Cancel")
    cancel_btn.bind('click', show_memos)
    menu <= cancel_btn
    
    del_btn = html.BUTTON("Delete", Class="danger")
    del_btn.bind('click', lambda ev:delete_memo(num))
    menu <= del_btn
    
    form <= menu
    
    doc.body.clear()
    doc <= form

def delete_memo(num):
    """Delete memo and go back to memos list"""
    num = int(num)
    del memos[num]
    store['brython-memo'] = json.dumps(memos)
    show_memos()

show_memos()
